package datacenter_client

import (
	"encoding/json"
	"errors"
	"fmt"
	"log"
	"metawant.greentech.com.cn/gaoyagang/gt-common/httplib"
)

type ItemHistoryData struct {
	ProjectId int `json:"project_id"`
	ItemName string  `json:"item_name"`
	Val      float64 `json:"val"`
	HTime    string  `json:"h_time"`
}

type MultiAddReq struct {
	List []ItemHistoryData `json:"list"`
}

type MultiAddBody struct {
	Code int              		`json:"code"`
	Msg  string             	`json:"msg"`
}

func (d *DcApiClient) MultiAddData (req MultiAddReq) (err error) {
	if len(req.List) == 0 {
		return errors.New("无可插入数据")
	}
	url := fmt.Sprintf("http://%s/api/dtgateway/v1/item-history/multi-add", d.ServerIp)
	h := httplib.Post(url)
	_, err = h.JSONBody(req)
	if err != nil {
		return
	}

	body, err := d.RequestMiddleProcess(h, fmt.Sprintf("%d", req.List[0].ProjectId))
	if err != nil {
		log.Println("GetItemHistory Error:", err)
		return
	}
	result := &ItemHistoryBody{}
	err = json.Unmarshal(body, &result)
	if err != nil {
		log.Println("GetItemHistory Error:", err)
		return
	}
	if result.Code != 200 {
		err = errors.New(result.Msg)
		return
	}
	return
}
